import sys
import time
import argparse
import shipyard_bp_utils as shipyard
from shipyard_coalesce import CoalesceClient

def create_pickle_file(response):
    # create artifacts folder to save response
    base_folder_name = shipyard.logs.determine_base_artifact_folder("coalesce")
    artifact_subfolder_paths = shipyard.logs.determine_artifact_subfolders(
        base_folder_name
    )
    shipyard.logs.create_artifacts_folders(artifact_subfolder_paths)
    # save sync response as variable
    shipyard.logs.create_pickle_file(
        artifact_subfolder_paths, "coalesce_response", response
    )

def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--access-token", dest="access_token", required=True)
    parser.add_argument("--environment-id", dest="environment_id", required=True)
    parser.add_argument("--job-id", dest="job_id", required=False, default=None)
    parser.add_argument(
        "--snowflake-username", dest="snowflake_username", required=True
    )
    parser.add_argument(
        "--snowflake-password", dest="snowflake_password", required=True
    )
    parser.add_argument("--snowflake-role", dest="snowflake_role", required=True)
    parser.add_argument(
        "--snowflake-warehouse", dest="snowflake_warehouse", required=True
    )
    parser.add_argument("--parallelism", dest="parallelism", required=False, default=16)
    parser.add_argument(
        "--include-nodes-selector", dest="include_nodes", required=False, default=None
    )
    parser.add_argument(
        "--exclude-nodes-selector", dest="exclude_nodes", required=False, default=None
    )
    parser.add_argument("--wait-for-completion",dest="wait_for_completion")
    parser.add_argument("--poke-interval",dest="poke_interval")

    return parser.parse_args()


def main():
    args = get_args()
    access_token = args.access_token
    sync_args = {
        "environment_id": args.environment_id,
        "job_id": None if args.job_id == '' else args.job_id,
        "snowflake_username": args.snowflake_username,
        "snowflake_password": args.snowflake_password,
        "snowflake_role": None if args.snowflake_role == '' else args.snowflake_role,
        "snowflake_warehouse": None if args.snowflake_role == '' else args.snowflake_warehouse,
        "parallelism": args.parallelism,
        "include_nodes_selector": None if args.include_nodes == '' else args.include_nodes,
        "exclude_nodes_selector": None if args.exclude_nodes == '' else args.exclude_nodes
    }
    client = CoalesceClient(access_token)

    try:
        response = client.trigger_sync(**sync_args)
    except Exception as e:
        client.logger.error(f"Error triggering sync: {e}")
        raise e

    if args.wait_for_completion == 'TRUE' and (0 < int(args.poke_interval) <= 60):
        run_id = response["runCounter"]
        status = client.determine_sync_status(run_id)

        while status not in (client.EXIT_CODE_FINAL_STATUS_COMPLETED,
                             client.EXIT_CODE_FINAL_STATUS_INCOMPLETE,
                             client.EXIT_CODE_FINAL_STATUS_CANCELLED,
            client.E
                             ):
            client.logger.info(f"Waiting {args.poke_interval} minute(s)...")
            time.sleep(int(args.poke_interval) * 60)
            status = client.determine_sync_status(run_id)

        sys.exit(status)
    elif args.wait_for_completion == 'TRUE':
        client.logger.error(
            "Poke interval must be between 1 and 60 minutes")
        sys.exit(client.EXIT_CODE_SYNC_INVALID_POKE_INTERVAL)
    else:
        create_pickle_file(response) # Backwards Compatibility: Ensures this code works with older versions of Check Sync Blueprint.



if __name__ == "__main__":
    main()
